/* LCD Include-Datei 
 *ACHTUNG PORT B !!!
 * Version fr 4x20 LCD 
 
 * Das LCD ist folgendermaen angeschlossen:
 * PB4     = RS
 * PB5     = Enable
 * PB0-PB3 = D4-D7
 * R/W ist n.c.
 *
 *
 */

#include <avr/io.h>
#include <util/delay.h>            


#define delay(us)  _delay_loop_2 (((F_CPU/4000)*us)/1000)		// wartet s

/**********************************************************
	* wartet Millisekunden
	* input:	Zeit in Millisekunden
	* return:	none
**********************************************************/
void delay_ms(uint16_t ms)
{
  for(uint16_t t=0; t<=ms; t++)
	_delay_ms(1); 
}

/**********************************************************
	* Enable Leitung togglen
	* input:	none
	* return:	none
**********************************************************/
void lcd_flash_e (void)
{
	PORTB = PORTB | ( 1<<DDB5 ) ;		// Enable = HIGH
	delay(100);							// kurz warten, damit Flanke entsteht
	PORTB = PORTB & ~( 1<<DDB5 ) ;		// Enable = LOW
	delay(100);							// Warten, bis der durch Enable angelegte Befehl hoffentlich verarbeitet wurde
}

/**********************************************************
	* Kommando oder Zeichen an Display senden: 
	* input:	rs=0 => Kommando, rs=1 => Zeichen
	* return:	none
**********************************************************/
void lcd_write (uint8_t data, uint8_t rs)   
{
	uint8_t dataBits ;
  
    if (rs)    				// write data (RS=1, RW=0) 
       dataBits=0x10;		// RS liegt an Pin 4 = B 0001 0000 = H 10
    else     				// write instruction (RS=0, RW=0) 
       dataBits=0; 			
	
    PORTB = dataBits | (data>>4);		// output high nibble first, zzgl. Zustand fr RS-Leitung
	lcd_flash_e ();    

	PORTB = dataBits | (data&0x0F);		// output low nibble, zzgl. RS-Leitung
	lcd_flash_e ();
}

/**********************************************************
	* Display loeschen
	* input:	none
	* return:	none
**********************************************************/
void lcd_cls (void)   
{
	
	lcd_write(0x01,0);   	// B 0000 0001 => Cursor Home
	delay(3000);			// dauert eine Weile, Wert ausprobiert
}

/**********************************************************
	* Zeichenausgabe
	* input: 	none
	* return:	none
**********************************************************/
void lcd_writechar ( char zeichen)   
{
  lcd_write (zeichen, 1);   
}

/**********************************************************
	* gibt eine Zeichenkette auf dem LCD aus
	* input:	none
	* return:	none
**********************************************************/
void lcd_writetext ( char *text)    
{
  uint8_t i = 0;
  while (text[i]!=0) 
  {
    lcd_writechar(text[i]);
    i++;
  }
}

/**********************************************************
	* Zeilenwechsel
	* input: 	none
	* return:	none
**********************************************************/
void lcd_gotoline (uint8_t zeile)    
{
	
	
	if (zeile == 1) lcd_write(0x80,0);   		// B 1000 0000 => DD-RAM Adress Set 1. Zeile/1.Spalte 
	if (zeile == 2) lcd_write(0x80+0x40,0);   	// B 1010 0000 => DD-RAM Adress Set 2. Zeile/1.Spalte 
	if (zeile == 3) lcd_write(0x80+0x14,0);   	// B 1010 0000 => DD-RAM Adress Set 3. Zeile/1.Spalte 
	if (zeile == 4) lcd_write(0x80+0x54,0);   	// B 1010 0000 => DD-RAM Adress Set 4. Zeile/1.Spalte 
	
	

}

/**********************************************************
	* Cursorpositionieren 
	* input: 	Zeile, Spalte
	* return:	none
**********************************************************/
void lcd_gotopos (uint8_t zeile, uint8_t spalte)    
{
	
	
	if (zeile == 1) lcd_write(0x80+0x00+spalte-1,0);   // DD-RAM Adress 1. Zeile + Spalte
	if (zeile == 2) lcd_write(0x80+0x40+spalte-1,0);   // DD-RAM Adress 2. Zeile + Spalte 
	if (zeile == 3) lcd_write(0x80+0x14+spalte-1,0);   // DD-RAM Adress 3. Zeile + Spalte 
	if (zeile == 4) lcd_write(0x80+0x54+spalte-1,0);   // DD-RAM Adress 4. Zeile + Spalte 
	
}

/**********************************************************
	* gibt eine int-Zahl auf dem Display aus. Integriert, damit auf string.h verzichtet werden kann
	* input: 	Zahl
			Anzahl der Stellen (fhrende Leerzeichen)
			space=0 => fhrende Nullen, 1 => fhrende Leerzeichen
	* return:	none
**********************************************************/
void lcd_writezahl (int32_t zahl, uint8_t pos, uint8_t spacer)       
{
   int32_t output;
   char sign = '0';
  
  pos--;       // Korrektur des Parameters fr Benutzerfreundlichkeit
  
  if (spacer) sign=' ';

  // negatives Vorzeichen oder Leerzeichen
  if (zahl < 0)
  {
	lcd_writechar ('-'); 
	zahl *= -1;		// positive Zahl
  }

  output = zahl;
  
  // Hunderttausender-Dezimalstelle oder Leerzeichen (Ausblenden fuehrender Nullen)
  if (zahl >= 100000) lcd_writechar (0x30 + output/100000); else if (pos >= 5) lcd_writechar (sign);
  output = output % 100000;

  // Zehntausender-Dezimalstelle oder Leerzeichen (Ausblenden fuehrender Nullen)
  if (zahl >= 10000) lcd_writechar (0x30 + output/10000); else if (pos >= 4) lcd_writechar (sign);
  output = output % 10000;

  // Tausender-Dezimalstelle oder Leerzeichen
  if (zahl >= 1000) lcd_writechar (0x30 + output/1000); else if (pos >= 3) lcd_writechar (sign);
  output = output % 1000;

  // Hunderter-Dezimalstelle oder Leerzeichen
  if (zahl >= 100) lcd_writechar (0x30 + output/100); else if (pos >= 2) lcd_writechar (sign);
  output = output % 100;

  // Zehner-Dezimalstelle oder Leerzeichen
  if (zahl >= 10) lcd_writechar (0x30 + output/10); else if (pos >= 1) lcd_writechar (sign);

  // einer Dezimalstelle oder Leerzeichen
  lcd_writechar (0x30 + output % 10);
}

/**********************************************************
	* Zuerst den 8-Bit-Modus aktivieren, Ist wichtig, falls LCD schon im 4-Bit Modus war und dann 
	  nach einem Programm-Reset vergeblich versucht wrde, den 4-Bit Modus erneut zu aktivieren.
	  dann in 4-Bit Modus umschalten.
	* input: 	none
	* return:	none
**********************************************************/
void lcd_set4bit (void)
{
    PORTB = 0x03;			// 8-Bit Modus aktivieren (B 0011 0000 = H 30, High Nibble => B 0011 = H 03
	lcd_flash_e ();    		// Datenbernahme
	delay_ms(5);			// sicherheitshalber warten da wir BUSY nicht berprfen knnen 

							// Das ganze 2x wiederholen:
	PORTB = 0x03;		
	lcd_flash_e ();    
	delay(2);			
	
	PORTB = 0x03;		
	lcd_flash_e ();    
	delay(2);				

	PORTB = 0x02;			// (B 0010 0000 = H 30, High Nibble => B 0010 = H 02)  in 4-Bit-Modus umschalten
	lcd_flash_e ();    
	delay(2);		
}

/**********************************************************
	* Portrichtung einstellen. Extern, damit man dies separat aufrufen kann, wenn bspw. LCD schon initialisiert ist und
	  per Watchdog ein Reset ausgelst wurde, man dann aber nicht das Display neu initialisieren mchte, damit es schneller weiter geht
	* input: 	none
	* return:	none
**********************************************************/
void lcd_port_ini (void)
{
	DDRB = 0x3F;			// setze Portrichtung am AVR (1 = Ausgang): 0011 1111 
}

/**********************************************************
	* Display initialisieren. Einmal als erstes aufrufen
	* input:	none
	* return:	none
**********************************************************/
void lcd_ini (void)
{
	lcd_port_ini();
	DDRB = 0x3F;			// setze Portrichtung (1 = Ausgang): 0011 1111 

	delay_ms(100);			// 40ms warten bis LCD wirklich bereit! Wir knnen nicht BUSY auswerten. 
							// Es gibt Displays die so lange brauchen (z. B. Optrex DMC20481)

	lcd_set4bit();			// ab jetzt 4-Bit-Modus
	lcd_write(0x28,0);		// B 0010 1000 => Function Set: 4Bit (kein 8-Bit Bus), zweizeiliges Display, 5x7 Dots/Zeichen (kein 5x10), RE-Bit aus
//	lcd_write(0x0F,0);		// B 0000 1111 => Display on/off: Display on, Cursor on, Blink on
	lcd_write(0x0C,0);		// B 0000 1100 => Display On/off: Display on, Cursor off, Blink off
	lcd_write(0x06,0);		// B 0000 0110 => Entry Mode Set: DD-RAM autom. inkr., Cursor bewegen
	lcd_cls();				// Display lschen
	
}


